@@ -26,6 +26,8 @@ class Socket(object):
             self.queue.task_done()
         except self.server.async['queue'].Empty:
             raise IOError()
+        if packets == [None]:
+            return []
         try:
             packets.append(self.queue.get(block=False))
             self.queue.task_done()
@@ -35,9 +37,10 @@ class Socket(object):
 
     def receive(self, pkt):
         """Receive packet from the client."""
-        self.server.logger.info('%s: Received packet %s with %s', self.sid,
-                                packet.packet_names[pkt.packet_type],
-                                pkt.data)
+        self.server.logger.info('%s: Received packet %s data %s',
+                                self.sid, packet.packet_names[pkt.packet_type],
+                                pkt.data if not isinstance(pkt.data, bytes)
+                                else '<binary>')
         if pkt.packet_type == packet.PING:
             self.last_ping = time.time()
             self.send(packet.Packet(packet.PONG, pkt.data))
@@ -58,9 +61,10 @@ class Socket(object):
             self.close(wait=False, abort=True)
             return
         self.queue.put(pkt)
-        self.server.logger.info('%s: Sending packet %s with %s', self.sid,
-                                packet.packet_names[pkt.packet_type],
-                                pkt.data)
+        self.server.logger.info('%s: Sending packet %s data %s',
+                                self.sid, packet.packet_names[pkt.packet_type],
+                                pkt.data if not isinstance(pkt.data, bytes)
+                                else '<binary>')
 
     def handle_get_request(self, environ, start_response):
         """Handle a long-polling GET request from the client."""
@@ -104,6 +108,10 @@ class Socket(object):
         """Upgrade the connection from polling to websocket."""
         if self.upgraded:
             raise IOError('Socket has been upgraded already')
+        if self.server.async['websocket'] is None or \
+                self.server.async['websocket_class'] is None:
+            # the selected async mode does not support websocket
+            return self.server._bad_request()
         websocket_class = getattr(self.server.async['websocket'],
                                   self.server.async['websocket_class'])
         ws = websocket_class(self._websocket_handler)
@@ -153,7 +161,7 @@ class Socket(object):
                 except:
                     break
 
-        writer_task = self.server.start_background_task(writer)
+        self.server.start_background_task(writer)
 
         self.server.logger.info(
             '%s: Upgrade to websocket successful', self.sid)
@@ -172,5 +180,5 @@ class Socket(object):
                 self.receive(pkt)
             except ValueError:
                 pass
-        self.close(wait=False, abort=True)
-        writer_task.join()
+        self.close(wait=True, abort=True)
+        self.queue.put(None)  # unlock the writer task so that it can exit
